@@ -32,17 +32,14 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define-module (gnucash report balsheet-eg))
-(use-modules (gnucash main))
+(use-modules (gnucash utilities))
 (use-modules (gnucash gnc-module))
 (use-modules (gnucash gettext))
 (use-modules (gnucash report eguile-gnc))
 (use-modules (gnucash report eguile-utilities))
+(use-modules (gnucash report eguile-html-utilities))
 
-(use-modules (ice-9 regex))  ; for regular expressions
-(cond-expand
-  (guile-2
-      (use-modules (ice-9 local-eval)))  ; for the-environment
-  (else ))
+(use-modules (ice-9 local-eval))  ; for the-environment
 (use-modules (srfi srfi-13)) ; for extra string functions
 
 (gnc:module-load "gnucash/report/report-system" 0)
@@ -50,38 +47,6 @@
 
 (define debugging? #f)
 
-;;; these could go into a separate module..........
-;;;
-;; Useful routines to use in the template
-(define (escape-html s1)
-  ;; convert string s1 to escape HTML special characters < > and &
-  ;; i.e. convert them to &lt; &gt; and &amp; respectively.
-  ;; Maybe there's a way to do this in one go... (but order is important)
-  (set! s1 (regexp-substitute/global #f "&" s1 'pre "&amp;" 'post))
-  (set! s1 (regexp-substitute/global #f "<" s1 'pre "&lt;" 'post))
-  (regexp-substitute/global #f ">" s1 'pre "&gt;" 'post))
-
-(define (nl->br str)
-  ;; replace newlines with <br>
-  (regexp-substitute/global #f "\n" str 'pre "<br />" 'post))
-
-(define (nbsp str)
-  ;; replace spaces with &nbsp; (non-breaking spaces)
-  ;; (yes, I know <nobr> is non-standard, but webkit splits e.g. "-£40.00" between
-  ;; the '-' and the '£' without it.)
-  (string-append "<nobr>" (regexp-substitute/global #f " " str 'pre "&nbsp;" 'post) "</nobr>"))
-
-(define (dump x) (escape-html (object->string x)))
-(define (ddump x) (display (dump x)))
-
-(define (string-repeat s n)
-  ;; return a string made of n copies of string s
-  ;; (there's probably a better way)
-  (let ((s2 ""))
-    (do ((i 1 (1+ i))) ((> i n))
-      (set! s2 (string-append s2 s)))
-    s2))
-
 (define (debug . args)
   (if debugging?
     (for arg in args do
@@ -95,34 +60,6 @@
   (display cols)
   (display "\">&nbsp;</td></tr>\n"))
 
-(define (empty-cells n)
-  ;; Display n empty table cells
-  (display (string-repeat "<td class=\"empty\"></td>" n)))
-
-(define (indent-cells n)
-  ;; Display n empty table cells with width attribute for indenting
-  ;; (the &nbsp;s are just there in case CSS isn't working)
-  (display (string-repeat "<td min-width=\"32\" class=\"indent\">&nbsp;&nbsp;</td>" n)))
-
-;; 'Safe' versions of cdr and cadr that don't crash
-;; if the list is empty  (is there a better way?)
-(define (safe-cdr l)
-  (if (null? l)
-    '()
-    (cdr l)))
-(define (safe-cadr l)
-  (if (null? l)
-    '()
-    (if (null? (cdr l))
-      '()
-      (cadr l))))
-
-(define (gnc:company-info key) ; this should be in business-utils.scm soon
-  ;; Access company info from key-value pairs for current book
-  (kvp-frame-get-slot-path-gslist
-    (qof-book-get-slots (gnc-get-current-book))
-    (append gnc:*kvp-option-path* (list gnc:*business-label* key))))
-
 (define (add-to-cc cc com num neg?)
   ; add a numeric and commodity to a commodity-collector,
   ; changing sign if required
@@ -389,11 +326,10 @@
          (opt-report-commodity (get-option commodities-page optname-report-commodity))
          (opt-price-source     (get-option commodities-page optname-price-source))
          (opt-show-foreign?    (get-option commodities-page optname-show-foreign))
-         (opt-report-title     (get-option general-page     optname-report-title))
-         (opt-date-tp          (gnc:timepair-end-day-time
+         (opt-report-title     (get-option general-page     gnc:optname-reportname))
+         (opt-date             (gnc:time64-end-day-time
                                  (gnc:date-option-absolute-time
                                    (get-option general-page optname-date))))
-         (opt-date-secs        (gnc:timepair->secs          opt-date-tp))
          (opt-columns          (get-option display-page     optname-columns))
          (opt-font-family      (get-option display-page     optname-font-family))
          (opt-font-size        (get-option display-page     optname-font-size))
@@ -433,16 +369,15 @@
 
          ;; exchange rates calculation parameters
          (exchange-fn
-           (gnc:case-exchange-fn opt-price-source opt-report-commodity opt-date-tp))
+           (gnc:case-exchange-fn opt-price-source opt-report-commodity opt-date))
          ; List of commodities (other than the local one) used
          ; so that exchange rate table can be displayed.
          ; xlist will become an association list of (comm . #t) pairs
          ; to avoid duplicates.
          (xlist '())
 
-         (coyname (or (gnc:company-info gnc:*company-name*) ""))
-
-         (css? (gnc-html-engine-supports-css))
+         ;; XXX I haven't found a way to get the book for which the report was opened here
+         (coyname (or (gnc:company-info (gnc-get-current-book) gnc:*company-name*) ""))
 
          (html #f))
 
@@ -467,14 +402,13 @@
 
     (define (foreignstyle item)
       ;; apply styling for amount in foreign currency
-      (if css?
-        (string-append "<span class=\"foreign\">" item "</span>"))
+        (string-append "<span class=\"foreign\">" item "</span>")
         (string-append "<small><i>" item "</i></small>"))
 
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     ;;; accrec-related routines
     ;;;
-    ;;; The accrec record stucture (defined above) and the following
+    ;;; The accrec record structure (defined above) and the following
     ;;; routines provide the basis for easy access to accounts
     ;;; from the main part of the report
 
@@ -506,9 +440,12 @@
              (and opt-omit-zb-accts?)
                   ;(> (accrec-depth accrec) 1))
              ;; Reason 1: zero Imbalance a/c
-             (string-prefix? (_ "Imbalance") (xaccAccountGetName (accrec-account accrec)))
+             ;; The line break in the next expressions will suppress comments as translator comments.
+             (string-prefix?
+               (_ "Imbalance") (xaccAccountGetName (accrec-account accrec)))
              ;; Reason 2: zero Orphan a/c
-             (string-prefix? (_ "Orphan") (xaccAccountGetName (accrec-account accrec))))))
+             (string-prefix?
+               (_ "Orphan") (xaccAccountGetName (accrec-account accrec))))))
 
     (define (flattened-acc-depth acc)
       ;; Accounts deeper than required get moved to the requested depth
@@ -532,7 +469,7 @@
                       (>= (gnc-account-get-current-depth (car account-list)) curr-depth))
                  (let* ((account (car account-list))
                         (comm    (xaccAccountGetCommodity account))
-                        (bal     (xaccAccountGetBalanceAsOfDate account opt-date-secs))
+                        (bal     (xaccAccountGetBalanceAsOfDate account opt-date))
                         (depth   (flattened-acc-depth account))
                         (treedepth 1)
                         ; Next account only qualifies as 'deeper' if we're not flattening
@@ -628,6 +565,12 @@
           (negstyle (nbsp mny-string)))
         (nbsp mny-string)))
 
+    (define (monetary-rounded mon)
+      (let ((c (gnc:gnc-monetary-commodity mon))
+            (a (gnc:gnc-monetary-amount mon)))
+        (gnc:make-gnc-monetary
+         c (gnc-numeric-convert a (gnc-commodity-get-fraction c) GNC-RND-ROUND))))
+
     (define (format-monetary mny)
       ;; Format the given gnc:monetary value according to opt-neg-format
       ;; If mny's currency isn't the same as that of the report,
@@ -641,10 +584,10 @@
         (if (not (gnc-commodity-equiv comm opt-report-commodity))
           (begin
             (if opt-show-foreign?
-              (set! answer (string-append (foreignstyle (neg-format (gnc:monetary->string mny) neg?)) "&nbsp;")))
+              (set! answer (string-append (foreignstyle (neg-format (gnc:monetary->string (monetary-rounded mny)) neg?)) "&nbsp;")))
             (set! mny (exchange-fn mny opt-report-commodity))))
         ; main currency - converted if necessary
-        (set! answer (string-append answer (neg-format (gnc:monetary->string mny) neg?)))
+        (set! answer (string-append answer (neg-format (gnc:monetary->string (monetary-rounded mny)) neg?)))
         answer))
 
     (define (format-comm-coll cc)
@@ -663,7 +606,7 @@
     (define (fmtmoney2 mny)
       ;; format a monetary amount in the given currency/commodity
       ;; !! this takes a gnc-monetary
-      (nbsp (gnc:monetary->string mny)))
+      (nbsp (gnc:monetary->string (monetary-rounded mny))))
 
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -677,13 +620,7 @@
     (set! html (eguile-file-to-string opt-template-file (the-environment)))
     (gnc:debug "balsheet-eg.scm - generated html:") (gnc:debug html)
     (gnc:report-finished)
-    (if css? ; return report as document or html, depending on version
-      html
-      (let ((document (gnc:make-html-document)))
-        (gnc:html-document-add-object! document html)
-        document))
-
-    ))
+    html))
 
 (gnc:define-report
   'version 1
