@@ -1,19 +1,33 @@
 ;; -*-scheme-*-
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; This program is free software; you can redistribute it and/or
+;; modify it under the terms of the GNU General Public License as
+;; published by the Free Software Foundation; either version 2 of
+;; the License, or (at your option) any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with this program; if not, contact:
+;;
+;; Free Software Foundation           Voice:  +1-617-542-5942
+;; 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
+;; Boston, MA  02110-1301,  USA       gnu@gnu.org
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
 
 ;; This is a sample guile report generator for GnuCash.
 ;; It illustrates the basic techniques used to create
 ;; new reports for GnuCash.
 
 (define-module (gnucash report hello-world))
-(use-modules (gnucash main)) ;; FIXME: delete after we finish modularizing.
+(use-modules (gnucash utilities)) 
 (use-modules (gnucash gnc-module))
 (use-modules (gnucash gettext))
 
-;; 'debug is deprecated and unused since guile 2
-(cond-expand
-  (guile-2 )
-  (else
-    (debug-enable 'debug)))
 (debug-enable 'backtrace)
 
 (gnc:module-load "gnucash/report/report-system" 0)
@@ -25,7 +39,7 @@
 (define (options-generator)    
   (let* ((options (gnc:new-options)) 
          ;; This is just a helper function for making options.
-         ;; See gnucash/src/app-utils/options.scm for details.
+         ;; See libgnucash/app-utils/options.scm for details.
          (add-option 
           (lambda (new-option)
             (gnc:register-option options new-option))))
@@ -79,8 +93,8 @@
       "c" (N_ "This is a string option.") (N_ "Hello, World")))
     
     ;; This is a date/time option. The user can pick a date and,
-    ;; possibly, a time. Times are stored as a pair
-    ;; (seconds . nanoseconds) measured from Jan 1, 1970, i.e.,
+    ;; possibly, a time. Times are stored as an integer specifying
+    ;; number of seconds measured from Jan 1, 1970, i.e.,
     ;; Unix time. The last option is false, so the user can only
     ;; select a date, not a time. The default value is the current
     ;; time.
@@ -88,7 +102,7 @@
      (gnc:make-date-option
       (N_ "Hello, World!") (N_ "Just a Date Option")
       "d" (N_ "This is a date option.")
-      (lambda () (cons 'absolute (cons (current-time) 0)))
+      (lambda () (cons 'absolute (current-time)))
       #f 'absolute #f ))
     
     ;; This is another date option, but the user can also select
@@ -97,7 +111,7 @@
      (gnc:make-date-option
       (N_ "Hello, World!") (N_ "Time and Date Option")
       "e" (N_ "This is a date option with time.")
-      (lambda () (cons 'absolute (cons (current-time) 0)))
+      (lambda () (cons 'absolute (current-time)))
       #t 'absolute #f ))
     
     (add-option
@@ -140,14 +154,14 @@
      (gnc:make-color-option
       (N_ "Hello, World!") (N_ "Background Color")
       "f" (N_ "This is a color option.")
-      (list #xf6 #xff #xdb 0)
+      (list #xf6 #xff #xdb #xff)
       255
       #f))
     (add-option
      (gnc:make-color-option
       (N_ "Hello, World!") (N_ "Text Color")
       "f" (N_ "This is a color option.")
-      (list #x00 #x00 #x00 0)
+      (list #x00 #x00 #x00 #xff)
       255
       #f))
     
@@ -253,13 +267,12 @@ option like this.")
 
     ;; these are samples of different date options. for a simple
     ;; date with day, month, and year but no time you should use
-    ;; gnc-print-date
-    (let ((time-string (strftime "%X" (localtime (current-time))))
-          (date-string (strftime "%x" (localtime (car date-val))))
-          (date-string2 (strftime "%x %X" (localtime (car date2-val))))
-          (rel-date-string (strftime "%x" (localtime (car rel-date-val))))
-          (combo-date-string
-           (strftime "%x" (localtime (car combo-date-val)))))
+    ;; qof-print-date
+    (let ((time-string (gnc-print-time64 (current-time) "%X"))
+          (date-string (gnc-print-time64 date-val "%x"))
+          (date-string2 (gnc-print-time64 date2-val "%x %X"))
+          (rel-date-string (gnc-print-time64 rel-date-val "%x"))
+          (combo-date-string (gnc-print-time64 combo-date-val "%x")))
 
       ;; Here's where we fill the report document with content.  We
       ;; do this by adding 'html objects' such as text, tables, and
@@ -332,56 +345,56 @@ or extending existing reports.")))
         (gnc:html-markup-p
          (gnc:html-markup/format
           (_ "For help on writing reports, or to contribute your brand \
-new, totally cool report, consult the mailing list %s.")
+new, totally cool report, consult the mailing list ~a.")
           (gnc:html-markup-anchor 
            "mailto:gnucash-devel@gnucash.org"
            (gnc:html-markup-tt "gnucash-devel@gnucash.org")))
-         (_ "For details on subscribing to that list, see &lt;http://www.gnucash.org/&gt;.")
-         (_ "You can learn more about writing scheme at &lt;http://www.scheme.com/tspl2d/&gt;."))
+         (_ "For details on subscribing to that list, see &lt;https://www.gnucash.org/&gt;.")
+         (_ "You can learn more about writing scheme at &lt;https://www.scheme.com/tspl2d/&gt;."))
 
         (gnc:html-markup-p
          (gnc:html-markup/format
-          (_ "The current time is %s.") 
+          (_ "The current time is ~a.") 
           (gnc:html-markup-b time-string)))
 
         (gnc:html-markup-p
          (gnc:html-markup/format
-          (_ "The boolean option is %s.")
+          (_ "The boolean option is ~a.")
           (gnc:html-markup-b (if bool-val (_ "true") (_ "false")))))
 
         (gnc:html-markup-p
          (gnc:html-markup/format
-          (_ "The multi-choice option is %s.")
+          (_ "The multi-choice option is ~a.")
           (gnc:html-markup-b (symbol->string mult-val))))
 
         (gnc:html-markup-p
          (gnc:html-markup/format
-          (_ "The string option is %s.") 
+          (_ "The string option is ~a.") 
           (gnc:html-markup-b string-val)))
 
         (gnc:html-markup-p
          (gnc:html-markup/format
-          (_ "The date option is %s.") 
+          (_ "The date option is ~a.") 
           (gnc:html-markup-b date-string)))
 
         (gnc:html-markup-p
          (gnc:html-markup/format
-          (_ "The date and time option is %s.") 
+          (_ "The date and time option is ~a.") 
           (gnc:html-markup-b date-string2)))
 
         (gnc:html-markup-p
          (gnc:html-markup/format 
-          (_ "The relative date option is %s.")
+          (_ "The relative date option is ~a.")
           (gnc:html-markup-b rel-date-string)))
 
         (gnc:html-markup-p
          (gnc:html-markup/format 
-          (_ "The combination date option is %s.")
+          (_ "The combination date option is ~a.")
           (gnc:html-markup-b combo-date-string)))
 
         (gnc:html-markup-p
          (gnc:html-markup/format 
-          (_ "The number option is %s.")
+          (_ "The number option is ~a.")
           (gnc:html-markup-b (number->string num-val))))
 
         ;; Here we print the value of the number option formatted as
@@ -392,10 +405,10 @@ new, totally cool report, consult the mailing list %s.")
         ;; it yourself -- it will be wrong in other locales.
         (gnc:html-markup-p 
          (gnc:html-markup/format
-          (_ "The number option formatted as currency is %s.")
+          (_ "The number option formatted as currency is ~a.")
           (gnc:html-markup-b
            (xaccPrintAmount
-            (gnc:make-gnc-numeric (inexact->exact num-val) 1)
+            (inexact->exact num-val)
             (gnc-default-print-info #f)))))))
 
       ;; you can add as many objects as you want.  Here's another 
