@@ -290,10 +290,9 @@
          (parent-balance-mode (get-option gnc:pagename-display
                                            optname-parent-balance-mode))
          (parent-total-mode
-	  (car
-	   (assoc-ref '((t #t) (f #f) (canonically-tabbed canonically-tabbed))
-		      (get-option gnc:pagename-display
-				  optname-parent-total-mode))))
+	  (assq-ref '((t . #t) (f . #f) (canonically-tabbed . canonically-tabbed))
+		    (get-option gnc:pagename-display
+				optname-parent-total-mode)))
          (show-zb-accts? (get-option gnc:pagename-display
 				     optname-show-zb-accts))
          (omit-zb-bals? (get-option gnc:pagename-display
@@ -458,23 +457,25 @@
 		)
           (if show-account-bals?
               (gnc:html-table-set-cell/tag!
-               build-table 0 (1+ cur-col) "number-header"
+               build-table 0 (+ cur-col account-cols) "number-header"
 	       (_ "Balance"))
               )
-          (let ((row 0))
-            (while (< row table-rows)
-              (let ((col 1))
-                (gnc:html-table-set-cell! build-table (1+ row) cur-col
-                                          (gnc:html-table-get-cell hold-table row 0))
-                (if show-account-bals?
-                    (while (< col hold-table-width)
-                      (if (gnc:html-table-get-cell hold-table row col)
-                          (gnc:html-table-set-cell!
-                           build-table (1+ row) (1+ cur-col)
-                           (gnc:html-table-get-cell hold-table row col)))
-                      (set! col (1+ col)))))
-              (set! row (1+ row))))
-          (set! cur-col (+ cur-col (if show-account-bals? 2 1)))
+	  (let ((row 0))
+	    (while (< row table-rows)
+		   (gnc:html-table-set-row-markup! build-table (+ row 1)
+						   (gnc:html-table-row-markup hold-table row))
+		   (let ((col 0))
+		     (while (< col hold-table-width)
+			    (gnc:html-table-set-cell!
+			     build-table (+ row 1) (+ cur-col col)
+			     (gnc:html-table-get-cell hold-table row col)
+			     )
+			    (set! col (+ col 1))
+			    )
+		     )
+		   (set! row (+ row 1))
+		   )
+	    )
 	  (set! cur-col (+ cur-col hold-table-width))
 	  (if show-account-notes?
 	      (begin
