@@ -373,7 +373,9 @@ developing over time"))
                                     (not (xaccTransGetIsClosingTxn
                                           (xaccSplitGetParent s))))
                                 (xaccSplitGetAmount s))))))))
-             accounts))
+             ;; all selected accounts (of report-specific type), *and*
+             ;; their descendants (of any type) need to be scanned.
+             (gnc:accounts-and-all-descendants accounts)))
 
           ;; Creates the <balance-list> to be used in the function
           ;; below.
@@ -469,9 +471,7 @@ developing over time"))
           ;; needed so as to amortize the cpu time properly.
           (gnc:report-percent-done 1)
           (set! commodity-list (gnc:accounts-get-commodities
-                                (append
-                                 (gnc:acccounts-get-all-subaccounts accounts)
-                                 accounts)
+                                (gnc:accounts-and-all-descendants accounts)
                                 report-currency))
           (set! exchange-fn (gnc:case-exchange-time-fn
                              price-source report-currency
