@@ -165,15 +165,14 @@
 	      (if show-comm?
 		  (gnc-commodity-table bal report-commodity exchange-fn)
 		  bal-sum))))
-    (car (assoc-ref
-	  (list
-	   (list 'entry balance)
-	   (list 'debit (if neg? #f balance))
-	   (list 'credit (if neg? balance #f))
-	   (list 'zero-q (if neg? #f (if balance #f #t)))
-	   (list 'debit-q (if neg? #f (if balance #t #f)))
-	   (list 'credit-q (if neg? #t #f)))
-	  req))))
+    (assq-ref (list
+               (cons 'entry balance)
+               (cons 'debit (if neg? #f balance))
+               (cons 'credit (if neg? balance #f))
+               (cons 'zero-q (if neg? #f (if balance #f #t)))
+               (cons 'debit-q (if neg? #f (if balance #t #f)))
+               (cons 'credit-q (if neg? #t #f)))
+              req)))
 
 ;; options generator
 (define (trial-balance-options-generator)
@@ -476,14 +475,7 @@
          ;; exchange rates calculation parameters
 	 (exchange-fn
 	  (gnc:case-exchange-fn price-source report-commodity end-date))
-	 (terse-period? #t)
-	 (period-for (if terse-period?
-			 (string-append " " (_ "for Period"))
-			 (format #f (string-append ", " (_ "~a to ~a"))
-				  (qof-print-date start-date-printable)
-				  (qof-print-date end-date))
-			 ))
-	 )
+	 (period-for (string-append " " (_ "for Period"))))
     
     (gnc:html-document-set-title! 
      doc (if (equal? report-variant 'current)
@@ -714,31 +706,17 @@
 		(set! parent-headings
 		      (apply append
 			     (map
-			      (if gnc:colspans-are-working-right
-				  (lambda (heading)
-				    (list 
-				     (gnc:make-html-table-cell/size/markup
-				      1 2 "th" heading)
-				     )
-				    )
-				  (lambda (heading)
-				    (list
-				     (gnc:make-html-table-cell/size/markup
-				      1 1 "th" heading)
-				     (gnc:html-make-empty-cell)
-				     )
-				    )
-				  )
+                              (lambda (heading)
+				(list
+				 (gnc:make-html-table-cell/size/markup 1 1 "th" heading)
+				 (gnc:html-make-empty-cell)))
 			      headings)
 			     )
 		      )
 		(gnc:html-table-append-row!
 		 build-table
 		 (append
-		  (if gnc:colspans-are-working-right
-		      (list (gnc:make-html-table-cell/size 1 account-cols #f))
-		      (gnc:html-make-empty-cells account-cols)
-		      )
+                  (gnc:html-make-empty-cells account-cols)
 		  parent-headings)
 		 )
 		(set! header-rows (+ header-rows 1))
@@ -1187,11 +1165,7 @@
 		 build-table
 		 "primary-subheading"
 		 (append
-		  (if gnc:colspans-are-working-right
-		      (list (gnc:make-html-table-cell/size
-			     1 (+ account-cols (* 2 is-col)) #f))
-		      (gnc:html-make-empty-cells (+ account-cols (* 2 is-col)))
-		      )
+                  (gnc:html-make-empty-cells (+ account-cols (* 2 is-col)))
 		  (list
 		   (tot-abs-amt-cell (if is-credit? tot-is is-debits))
 		   (tot-abs-amt-cell (if is-credit? is-credits tot-is))
@@ -1217,8 +1191,6 @@
 	  (gnc:report-percent-done 100)
 	  
 	  ;; if sending the report to a file, do so now
-	  ;; however, this still doesn't seem to get around the
-	  ;; colspan bug... cf. gnc:colspans-are-working-right
 	  (if filename
 	      (let* ((port (open-output-file filename)))
                 (gnc:display-report-list-item
