@@ -14,7 +14,7 @@
 ;; Kennzahlen (from txf-de_DE.scm) as assigned to the different
 ;; accounts, and will write it to some XML file as required by
 ;; e.g. the Winston software
-;; http://www.felfri.de/winston/schnittstellen.htm
+;; https://www.felfri.de/winston/schnittstellen.htm
 ;;
 ;; This file might still contain a lot of US-TXF related stuff. This
 ;; can surely be thrown out once someone was able to actually use this
@@ -40,24 +40,42 @@
 ;; Optionally prints brief or full account names
 ;;
 ;; NOTE: setting of specific dates is squirly! and seems to be
-;; current-date dependant!  Actually, time of day dependant!  Just
+;; current-date dependent!  Actually, time of day dependent!  Just
 ;; after midnight gives different dates than just before!  Referencing
 ;; all times to noon seems to fix this.  Subtracting 1 year sometimes
 ;; subtracts 2!  see "(to-value"
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; This program is free software; you can redistribute it and/or
+;; modify it under the terms of the GNU General Public License as
+;; published by the Free Software Foundation; either version 2 of
+;; the License, or (at your option) any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with this program; if not, contact:
+;;
+;; Free Software Foundation           Voice:  +1-617-542-5942
+;; 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
+;; Boston, MA  02110-1301,  USA       gnu@gnu.org
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
 
 ;; depends must be outside module scope -- and should eventually go away.
 
 (define-module (gnucash report taxtxf-de_DE))
-(use-modules (gnucash main)) ;; FIXME: delete after we finish modularizing.
+(use-modules (gnucash utilities)) 
 (use-modules (srfi srfi-1))
-(use-modules (gnucash printf))
 (use-modules (gnucash core-utils)) ; for gnc:version
 (use-modules (gnucash gettext))
 
 (use-modules (gnucash gnc-module))
 (gnc:module-load "gnucash/tax/de_DE" 0)
 (gnc:module-load "gnucash/report/report-system" 0)
-
+(gnc:module-load "gnucash/engine" 0)
 
 (define reportname (N_ "Tax Report / TXF Export"))
 
@@ -72,19 +90,18 @@
 
 (define levelx-collector (make-level-collector MAX-LEVELS))
 
-(define today (timespecCanonicalDayTime
-               (cons (current-time) 0)))
+(define today (time64CanonicalDayTime (current-time)))
 
 (define bdtm
-  (let ((result (gnc:timepair->date today)))
+  (let ((result (gnc-localtime today)))
     (set-tm:mday result 16)             ; 16
     (set-tm:mon result 3)               ; Apr
     (set-tm:isdst result -1)
     result))
 
-(define tax-day (cons (car (mktime bdtm)) 0))
+(define tax-day (gnc-mktime bdtm))
 
-(define after-tax-day (gnc:timepair-later tax-day today))
+(define after-tax-day (< tax-day today))
 
 (define (make-split-list account split-filter-pred)
   (reverse (filter split-filter-pred
@@ -92,14 +109,14 @@
 
 ;; returns a predicate that returns true only if a split is
 ;; between early-date and late-date
-(define (split-report-make-date-filter-predicate begin-date-tp
-                                                 end-date-tp)
+(define (split-report-make-date-filter-predicate begin-date
+                                                 end-date)
   (lambda (split) 
-    (let ((tp
-           (gnc-transaction-get-date-posted
+    (let ((t
+           (xaccTransGetDate
             (xaccSplitGetParent split))))
-      (and (gnc:timepair-ge-date tp begin-date-tp)
-           (gnc:timepair-le-date tp end-date-tp)))))
+      (and (>= t begin-date)
+           (<= t end-date)))))
 
 ;; This is nearly identical to, and could be shared with
 ;; display-report-list-item in report.scm. This adds warn-msg parameter
@@ -303,10 +320,10 @@
         (let* ((type (xaccAccountGetType account))
                (code (gnc:account-get-txf-code account))
                (date-str (if date
-                             (strftime "%d.%m.%Y" (localtime (car date)))
+                             (gnc-print-time64 date "%d.%m.%Y")
                              #f))
                (x-date-str (if x-date
-                               (strftime "%d.%m.%Y" (localtime (car x-date)))
+                               (gnc-print-time64 x-date "%d.%m.%Y")
                                #f))
                ;; Only formats 1,3 implemented now! Others are treated as 1.
                (format (gnc:get-txf-format code (eq? type ACCT-TYPE-INCOME)))
@@ -391,7 +408,7 @@
 (define (render-level-x-account table level max-level account lx-value
                                 suppress-0 full-names txf-date)
   (let* ((account-name (if txf-date	; special split
-                           (strftime "%d.%m.%Y" (localtime (car txf-date)))
+                           (gnc-print-time64 txf-date "%d.%m.%Y")
                            (if (or full-names (equal? level 1))
                                (gnc-account-get-full-name account)
                                (xaccAccountGetName account))))
@@ -471,7 +488,7 @@
   (gnc:report-starting reportname)
   (let* ((from-value (gnc:date-option-absolute-time 
                       (get-option gnc:pagename-general "From")))
-         (to-value (gnc:timepair-end-day-time
+         (to-value (gnc:time64-end-day-time
                     (gnc:date-option-absolute-time 		       
                      (get-option gnc:pagename-general "To"))))
          (alt-period (get-option gnc:pagename-general "Alternate Period"))
@@ -488,6 +505,11 @@
                                 (validate (reverse 
                                            (gnc-account-get-children-sorted
                                             (gnc-get-current-root-account))))))
+         (book (if selected-accounts
+                   (gnc-account-get-book (if (pair? selected-accounts)
+                                             (car selected-accounts)
+                                             selected-accounts))
+                   #f))
          (generations (if (pair? selected-accounts)
                           (apply max (map (lambda (x) (num-generations x 1))
                                           selected-accounts))
@@ -497,8 +519,8 @@
 	 (work-done 0)
 
          ;; Alternate dates are relative to from-date
-         (from-date (gnc:timepair->date from-value))
-         (from-value (gnc:timepair-start-day-time
+         (from-date (gnc-localtime from-value))
+         (from-value (gnc:time64-start-day-time
                       (let ((bdtm from-date))
                         (if (member alt-period 
                                     '(last-year 1st-last 2nd-last
@@ -527,9 +549,9 @@
                               ((4th-est 4th-last) ; Oct 1
                                (set-tm:mon bdtm 9))))
                         (set-tm:isdst bdtm -1)
-                        (cons (car (mktime bdtm)) 0))))
+                        (gnc-mktime bdtm))))
 
-         (to-value (gnc:timepair-end-day-time
+         (to-value (gnc:time64-end-day-time
                     (let ((bdtm from-date))
                       (if (member alt-period 
                                   '(last-year 1st-last 2nd-last
@@ -553,8 +575,8 @@
                             ((3rd-est 3rd-last) ; Aug 31
                              (set-tm:mon bdtm 7))
                             ((4th-est 4th-last last-year) ; Dec 31
-                             (set-tm:mon bdtm 11))
-                            (else (set! bdtm (gnc:timepair->date to-value))))
+                             (set-tm:mon bdtm 11)) 
+                            (else (set! bdtm (gnc-mktime to-value))))
                           ;; Tax quaters equal Real quarters
                           (case alt-period
                             ((1st-est 1st-last) ; Mar 31
@@ -568,9 +590,9 @@
                             ((4th-est 4th-last last-year) ; Dec 31
                              (set-tm:mon bdtm 11))
                             (else 
-                             (set! bdtm (gnc:timepair->date to-value)))))
+                             (set! bdtm (gnc-mktime to-value)))))
                       (set-tm:isdst bdtm -1)
-                      (cons (car (mktime bdtm)) 0))))
+                      (gnc-mktime bdtm))))
 
          (txf-feedback-str-lst '())
          (doc (gnc:make-html-document))
@@ -583,8 +605,8 @@
                (txf-special-split? (gnc:account-get-txf-code account)))
           (let* 
               ((full-year?
-                (let ((bdto (localtime (car to-value)))
-                      (bdfrom (localtime (car from-value))))
+                (let ((bdto (gnc-localtime to-value))
+                      (bdfrom (gnc-localtime from-value)))
                   (and (equal? (tm:year bdto) (tm:year bdfrom))
                        (equal? (tm:mon bdfrom) 0)
                        (equal? (tm:mday bdfrom) 1)
@@ -593,23 +615,23 @@
                ;; Adjust dates so we get the final Estimated Tax
                ;; paymnent from the right year
                (from-est (if full-year?
-                             (let ((bdtm (gnc:timepair->date
-                                          (timespecCanonicalDayTime
+                             (let ((bdtm (gnc-localtime
+                                          (time64CanonicalDayTime
                                            from-value))))
                                (set-tm:mday bdtm 1) ; 01
                                (set-tm:mon bdtm 2) ; Mar
                                (set-tm:isdst bdtm -1)
-                               (cons (car (mktime bdtm)) 0))
+                               (gnc-mktime bdtm))
                              from-value))
                (to-est (if full-year?
-                           (let* ((bdtm (gnc:timepair->date
-                                         (timespecCanonicalDayTime
+                           (let* ((bdtm (gnc-localtime
+                                         (time64CanonicalDayTime
                                           from-value))))
                              (set-tm:mday bdtm 28) ; 28
                              (set-tm:mon bdtm 1) ; Feb
                              (set-tm:year bdtm (+ (tm:year bdtm) 1))
                              (set-tm:isdst bdtm -1)
-                             (cons (car (mktime bdtm)) 0))
+                             (gnc-mktime bdtm))
                            to-value)))
             (list from-est to-est full-year?))
           #f))
@@ -626,12 +648,12 @@
                      (+ 1 level)
                      level)))
         (map (lambda (spl) 
-               (let* ((date (gnc-transaction-get-date-posted
+               (let* ((date (xaccTransGetDate
                              (xaccSplitGetParent spl)))
                       (amount (xaccSplitGetAmount spl))
                       ;; TurboTax 1999 and 2000 ignore dates after Dec 31
                       (fudge-date (if (and full-year? 
-                                           (gnc:timepair-lt to-value date))
+                                           (< to-value date))
                                       to-value
                                       date)))
                  (if tax-mode?
@@ -745,19 +767,16 @@
             ;; Ignore
             '())))
 
-    (let ((from-date  (strftime "%d.%m.%Y" (localtime (car from-value))))
-          (to-date    (strftime "%d.%m.%Y" (localtime (car to-value))))
-	  (to-year    (strftime "%Y" (localtime (car to-value))))
-          (today-date (strftime "%d.%m.%Y" 
-                                (localtime 
-                                 (car (timespecCanonicalDayTime
-                                       (cons (current-time) 0))))))
-	  (tax-nr (or 
-		   (kvp-frame-get-slot-path-gslist
-		    (qof-book-get-slots (gnc-get-current-book))
-		    (append gnc:*kvp-option-path*
-			    (list gnc:*tax-label* gnc:*tax-nr-label*)))
-		   ""))
+    (let ((from-date  (gnc-print-time64 from-value "%d.%m.%Y"))
+          (to-date    (gnc-print-time64 to-value "%d.%m.%Y"))
+	  (to-year    (gnc-print-time64 to-value "%Y"))
+          (today-date (gnc-print-time64 (time64CanonicalDayTime (current-time))
+                                        "%d.%m.%Y"))
+	  (tax-nr (unless book
+                      (or
+                       (gnc:option-get-value book gnc:*tax-label* gnc:*tax-nr-label*)
+                       "")
+                      ""))
 	  )
 
       ;; Now, the main body
@@ -829,7 +848,7 @@
                    "center"
                    (gnc:html-markup-p
                     (gnc:html-markup/format
-                     (_ "Period from %s to %s") from-date to-date)))))
+                     (_ "Period from ~a to ~a") from-date to-date)))))
             
             (gnc:html-document-add-object!
              doc (gnc:make-html-text
