@@ -82,7 +82,7 @@
 ;; option may have.
 ;; 
 ;; The list of accounts which are to be placed in the
-;; gnc:html-acct-table object can be controled with the
+;; gnc:html-acct-table object can be controlled with the
 ;; gnc:make-html-acct-table/accts, gnc:make-html-acct-table/env/accts,
 ;; and gnc:html-table-add-accts!  functions.  But you should only use
 ;; one of these methods to add accounts.
@@ -129,14 +129,14 @@
 ;;         sorting function.  #f means to preform no sorting.  the
 ;;         default sorting function is gnc:account-code-less-p.
 ;; 
-;;     start-date: timepair
+;;     start-date: time64
 ;; 
 ;;         the starting date of the reporting period over which to
 ;;         report balances for this account.  if start-date is #f,
 ;;         will be no limit on how early a counted transaction may
 ;;         ocurr.
 ;; 
-;;     end-date: timepair
+;;     end-date: time64
 ;; 
 ;;         the ending date of the reporting period over which to
 ;;         report balances for this account.  if end-date is #f, there
@@ -338,7 +338,7 @@
 ;;         account-depth when depth-limit-behavior is 'flatten.
 ;;         unlike in gnc:html-build-acct-table, the first level of
 ;;         accounts is level 0. this means that display-depth is also
-;;         the number of empty cells which should preceed the account
+;;         the number of empty cells which should preced the account
 ;;         name in the gnc:html-table being generated.
 ;; 
 ;;     indented-depth: integer
@@ -368,7 +368,7 @@
 ;;         value of display-tree-depth, plus indent plus zero, if
 ;;         parent-account-subotal-mode is not 'canonically-tabbed, or,
 ;;         if parent-account-subtotal-mode is 'canonically-tabbed,
-;;         plus one.  dont you just love english?
+;;         plus one.  don't you just love english?
 ;; 
 ;;     account-colspan: integer
 ;; 
@@ -440,7 +440,7 @@
 ;; widget.  While, when selecting whole subtrees of accounts, this may
 ;; be tedious, this really is a GUI problem.  The ideal solution would
 ;; be to give the Account selection widget a "recursively select"
-;; option which selects (i.e., hilights) both the account selected and
+;; option which selects (i.e., highlights) both the account selected and
 ;; all its subaccounts.  Note that, as a worst-case workaround, the
 ;; user could always use the spacebar and arrow keys to select entire
 ;; subtrees rather rapidly.  It order to make this shortcoming as
@@ -495,32 +495,27 @@
 ;; user.  This class simply maps its contents to the html-table.
 ;; 
 
+(use-modules (srfi srfi-9))
+
 ;; this is to work around a bug in the HTML export sytmem
 ;; which causes COLSPAN= attributes not to be exported (!!)
-(define gnc:colspans-are-working-right #f)
+(define gnc:colspans-are-working-right
+  ;; should be deprecated
+  #f)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;  <html-acct-table> class
 ;;  utility class for generating account tables
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define <html-acct-table>
-  (make-record-type "<html-acct-table>"
-		    '(matrix ;; an html-table
-		      env    ;; an alist
-		      )))
-
-(define gnc:html-acct-table? 
-  (record-predicate <html-acct-table>))
-
-(define gnc:_make-html-acct-table_
-  (record-constructor <html-acct-table>))
+(define-record-type <html-acct-table>
+  (gnc:_make-html-acct-table_ matrix env)
+  gnc:html-acct-table?
+  (matrix gnc:_html-acct-table-matrix_ gnc:_html-acct-table-set-matrix!_)
+  (env gnc:_html-acct-table-env_ gnc:_html-acct-table-set-env!_))
 
 (define (gnc:make-html-acct-table)
-  (gnc:_make-html-acct-table_
-   (gnc:make-html-table) ;; matrix
-   #f                    ;; env
-   ))
+  (gnc:_make-html-acct-table_ (gnc:make-html-table) #f))
 
 (define (gnc:make-html-acct-table/env env)
   (let ((acct-table (gnc:make-html-acct-table)))
@@ -536,18 +531,6 @@
     (gnc:html-acct-table-add-accounts! acct-table accts)
     acct-table))
 
-(define gnc:_html-acct-table-matrix_
-  (record-accessor <html-acct-table> 'matrix))
-
-(define gnc:_html-acct-table-set-matrix!_
-  (record-modifier <html-acct-table> 'matrix))
-
-(define gnc:_html-acct-table-env_
-  (record-accessor <html-acct-table> 'env))
-
-(define gnc:_html-acct-table-set-env!_
-  (record-modifier <html-acct-table> 'env))
-
 ;; some useful predicates to export
 (define (gnc:account-code-less-p a b)
   (string<? (xaccAccountGetCode a)
@@ -577,14 +560,21 @@
 
   ;; helper to plop <env> in the next available env cell
   (define (add-row env)
-    (let ((html-table (gnc:_html-acct-table-matrix_ acct-table)))
+    (let* ((html-table (gnc:_html-acct-table-matrix_ acct-table))
+           (row (gnc:html-table-num-rows html-table)))
       (gnc:html-table-set-cell!
        html-table
-       (gnc:html-table-num-rows html-table)
+       row
        0
        env)
+      row
       )
     )
+
+  ;; Add more stuff to an existing row
+  (define (append-to-row row env)
+    (gnc:html-acct-table-set-row-env! acct-table row
+      (append (gnc:html-acct-table-get-row-env acct-table row) env)))
   
   (let* ((env (gnc:_html-acct-table-env_ acct-table))
 	 ;; establish all input parameters and their defaults 
@@ -599,7 +589,7 @@
 		   (if (equal? pred #t) gnc:account-code-less-p pred)))
 	 (start-date (get-val env 'start-date))
 	 (end-date (or (get-val env 'end-date)
-		       (cons 'absolute (cons (current-time) 0))))
+		       (gnc:get-today)))
 	 (report-commodity (or (get-val env 'report-commodity)
 			       (gnc-default-report-currency)))
          ;; BUG: other code expects a real function here, maybe
@@ -607,7 +597,6 @@
 	 (exchange-fn (or (get-val env 'exchange-fn)
                           #f))
          (get-balance-fn (or (get-val env 'get-balance-fn) #f))
-         ;;'weighted-average))
 	 (column-header (let ((cell (get-val env 'column-header)))
 			  (if (equal? cell #t)
 			      (gnc:make-html-table-cell "Account name")
@@ -624,6 +613,7 @@
 			       (list 'str (_ "Closing Entries"))
 			       (list 'cased #f)
 			       (list 'regexp #f)
+			       (list 'closing #t)
 			       )
 			      ))
 	 (adjusting-pattern (or (get-val env 'adjusting-pattern)
@@ -668,9 +658,6 @@
                     (acct-comm (xaccAccountGetCommodity acct))
                     (shares (xaccSplitGetAmount split))
                     (hash (hash-ref hash-table guid)))
-;                (gnc:debug "Merging split for " (xaccAccountGetName acct) " for "
-;                           (gnc-commodity-numeric->string acct-comm shares)
-;                           " into hash entry " hash)
                (if (not hash)
                    (begin (set! hash (gnc:make-commodity-collector))
                           (hash-set! hash-table guid hash)))
@@ -728,13 +715,12 @@
       ;; ( acct . balance ) cells
       (define (get-balance acct-balances acct)
 	(let ((this-collector (gnc:make-commodity-collector)))
-	  (gnc-commodity-collector-merge 
-	   this-collector 
+	  (this-collector
+           'merge
 	   (or (hash-ref acct-balances (gncAccountGetGUID acct))
 	       ;; return a zero commodity collector
-	       (gnc:make-commodity-collector)
-	       )
-	   )
+	       (gnc:make-commodity-collector))
+	   #f)
 	  this-collector
 	  )
 	)
@@ -750,9 +736,9 @@
 	(let ((this-collector (gnc:make-commodity-collector)))
 	  ;; get the balance of the parent account and stick it on the collector
 	  ;; that nice shiny *NEW* collector!!
-	  (gnc-commodity-collector-merge this-collector (get-balance acct-balances account))
+	  (this-collector 'merge (get-balance acct-balances account) #f)
 	  (for-each
-	   (lambda (x) (if x (gnc-commodity-collector-merge this-collector x)))
+	   (lambda (x) (if x (this-collector 'merge x #f)))
 	   (gnc:account-map-descendants
 	    (lambda (a)
 	      (get-balance acct-balances a ))
@@ -764,6 +750,7 @@
 	     (if (integer? depth-limit)
 		 (min (- depth-limit 1) logi-depth)
 		 logi-depth))
+            (row-added? #f)
 	    )
 	
 	(for-each
@@ -834,6 +821,8 @@
 			     (and (equal? label-mode 'name)
 				  (gnc:make-html-text account-name))
 			     ))
+                  (row #f)
+                  (children-displayed? #f)
 		  )
 
 	     (set! acct-depth-reached (max acct-depth-reached acct-depth))
@@ -860,18 +849,22 @@
 					(+ disp-depth indent))
 				  )
 				 ))
-		   (add-row row-env)
+		   (set! row (add-row row-env))
 		   )
 		 )
 	     ;; Recurse:
-	     ;; Dive into an account even if it isnt selected!
+	     ;; Dive into an account even if it isn't selected!
 	     ;; why? because some subaccts may be selected.
-	     (traverse-accounts! subaccts
-				 (+ acct-depth 1)
-				 (if (use-acct? acct)
-				     (+ logi-depth 1)
-				     logi-depth)
-				 new-balances)
+	     (set! children-displayed?
+	           (traverse-accounts! subaccts
+				       (+ acct-depth 1)
+				       (if (use-acct? acct)
+				           (+ logi-depth 1)
+				           logi-depth)
+				       new-balances))
+
+	     ;; record whether any children were displayed
+	     (if row (append-to-row row (list (list 'children-displayed? children-displayed?))))
 
 	     ;; after the return from recursion: subtotals
 	     (or (not (use-acct? acct))
@@ -881,7 +874,7 @@
 		      (equal? zero-mode 'omit-leaf-acct))
 		 ;; ignore use-acct for subtotals...?
 		 ;; (not (use-acct? acct))
-		 (null? subaccts)
+		 (not children-displayed?)
 		 (let* ((lbl-txt (gnc:make-html-text (_ "Total") " ")))
 		   (apply gnc:html-text-append! lbl-txt
 			  (gnc:html-text-body label))
@@ -903,12 +896,14 @@
 		   (add-row row-env)
 		   )
 		 )
+	     (if (or row-added? children-displayed? row) (set! row-added? #t))
 	     )) ;; end of (lambda (acct) ...)
 	 ;; lambda is applied to each item in the (sorted) account list
 	 (if less-p
 	     (sort accts less-p)
 	     accts)
 	 ) ;; end of for-each
+	 row-added?
 	)
       ) ;; end of definition of traverse-accounts!
 
@@ -916,14 +911,6 @@
     (traverse-accounts! toplvl-accts 0 0
                         (calculate-balances accounts start-date end-date get-balance-fn))
     
-    ;; set the column-header colspan
-    (if gnc:colspans-are-working-right
-	(if (gnc:html-table-cell? column-header)
-	    (gnc:html-table-cell-set-colspan! column-header
-					      (+ disp-depth-reached 1 indent))
-	    )
-	)
-    
     ;; now set the account-colspan entries
     ;; he he... (let ((x 0)) (while (< x 5) (display x) (set! x (+ x 1))))
     ;; now I know how to loop in scheme... yay!
@@ -977,6 +964,7 @@
   (gnc:html-table-num-rows (gnc:_html-acct-table-matrix_ acct-table)))
 
 (define (gnc:html-acct-table-num-cols acct-table)
+  (issue-deprecation-warning "gnc:html-acct-table-num-cols is unused.")
   (- (gnc:html-table-num-columns (gnc:_html-acct-table-matrix_ acct-table)) 1))
 
 (define (gnc:html-acct-table-get-cell acct-table row col)
@@ -1002,44 +990,47 @@
 (define (gnc:html-acct-table-set-row-env! acct-table row env)
   (gnc:html-acct-table-set-cell! acct-table row -1 env))
 
-(define (gnc:html-acct-table-append-row acct-table objects)
+(define (gnc:html-acct-table-append-row! acct-table newrow)
+  (issue-deprecation-warning "gnc:html-acct-table-append-row! is unused.")
   (gnc:html-table-append-row!
    (gnc:_html-acct-table-matrix_ acct-table)
    (map
     (lambda (x) (gnc:make-html-table-cell (list x)))
-    objects)))
+    newrow)))
 
 (define (gnc:html-acct-table-prepend-row! acct-table newrow)
+  (issue-deprecation-warning "gnc:html-acct-table-prepend-row! is unused.")
   (gnc:html-table-prepend-row!
    (gnc:_html-acct-table-matrix_ acct-table)
    (map
     (lambda (x) (gnc:make-html-table-cell (list x)))
-    objects)))
+    newrow)))
 
-(define (gnc:html-acct-table-append-col acct-table objects)
+(define (gnc:html-acct-table-append-col! acct-table newcol)
+  (issue-deprecation-warning "gnc:html-acct-table-append-col! is unused.")
   (gnc:html-table-append-col!
    (gnc:_html-acct-table-matrix_ acct-table)
    (map
     (lambda (x) (gnc:make-html-table-cell (list x)))
-    objects)))
+    newcol)))
 
 (define (gnc:html-acct-table-prepend-col! acct-table newrow)
+  (issue-deprecation-warning "gnc:html-acct-table-prepend-col! is unused.")
   (gnc:html-table-prepend-col!
    (gnc:_html-acct-table-matrix_ acct-table)
    (map
     (lambda (x) (gnc:make-html-table-cell (list x)))
-    objects)))
+    newcol)))
 
 (define (gnc:html-acct-table-remove-last-row! acct-table)
+  (issue-deprecation-warning "gnc:html-acct-table-remove-last-row! is unused.")
   (gnc:html-table-remove-last-row! (gnc:_html-acct-table-matrix_ acct-table)))
 
-;; don't think we need this.
-;;(define (gnc:identity i) i)
-
 (define (gnc:html-acct-table-render acct-table doc)
   ;; this will be used if we ever decide to let the utility object
   ;; render a document by calling thunks registered in the row-envs...
   ;; but, for now, this (optional) feature is left unimplemented...
+  (issue-deprecation-warning "gnc:html-acct-table-render is unused.")
   #f
   )
 
@@ -1048,9 +1039,10 @@
 ;; 
 
 (define (gnc:html-make-nbsps n)
-  (if (> n 0)
-      (string-append "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" (gnc:html-make-nbsps (- n 1)))
-      ""))
+  (let lp ((n n) (res '()))
+    (if (positive? n)
+        (lp (1- n) (cons "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" res))
+        (string-join res ""))))
 
 ;; Stylesheets define the following cell styles which these functions
 ;; use: "text-cell" "total-label-cell" "number-cell"
@@ -1078,13 +1070,9 @@
          amount-colspan    ;; defaults to one
          amount-markup)    ;; optional
   (let* ((lbl-depth (or label-depth 0))
-	 (lbl-colspan (if gnc:colspans-are-working-right
-			  (or label-colspan 1)
-			  1))
+	 (lbl-colspan 1)
 	 (amt-depth (or amount-depth (+ lbl-depth lbl-colspan)))
-	 (amt-colspan (if gnc:colspans-are-working-right
-			  (or amount-colspan 1)
-			  1))
+	 (amt-colspan 1)
 	 (tbl-width (or table-width (+ amt-depth amt-colspan)))
 	 (row
 	  (append
@@ -1096,7 +1084,7 @@
 		 1 1 (gnc:make-html-text (gnc:html-make-nbsps lbl-depth)) label))
 	    )
 	   (gnc:html-make-empty-cells             ;; padding after label
-            (+ (- amt-depth (/ tbl-width 2))
+            (+ (- amt-depth (floor (/ tbl-width 2)))
                (if total-rule? -1 0)
                )
             )
@@ -1129,25 +1117,22 @@
   ;; be a bit redundant, i beleive that it makes the report more
   ;; readable.
   (let* ((table (gnc:make-html-table))
-	 )
-    (gnc-commodity-collector-map
-     amount
-     (lambda (curr val)
-       (let ((bal (gnc:make-gnc-monetary curr val)))
-	 (gnc:html-table-append-row!
-	  table
-	  (list
-	   ;; add the account balance in the respective commodity
-	   (gnc:make-html-table-cell/markup
-	    "number-cell" bal)
-	   ;; add the account balance in the report commodity
-	   (gnc:make-html-table-cell/markup
-	    "number-cell" (exchange-fn bal report-commodity))
-	   )
-	  )
-	 )))
-    table)
-  )
+         (spacer (gnc:make-html-table-cell))
+         (list-of-balances (amount 'format gnc:make-gnc-monetary #f)))
+    (gnc:html-table-cell-set-style! spacer "td"
+                                    'attribute (list "style" "min-width: 1em"))
+    (for-each
+     (lambda (bal)
+       (gnc:html-table-append-row!
+	table (list (gnc:make-html-table-cell/markup "number-cell" bal)
+                    spacer
+	            (gnc:make-html-table-cell/markup
+                     "number-cell" (exchange-fn bal report-commodity)))))
+     list-of-balances)
+    (gnc:html-table-set-style! table "table"
+                               'attribute (list "style" "width:100%; max-width:20em")
+                               'attribute (list "cellpadding" "0"))
+    table))
 
 ;; 
 ;; This function adds all the lines from a gnc:html-acct-table to a
@@ -1204,96 +1189,52 @@
 		  (parent-acct-bal-mode
 		   (or (get-val env 'parent-account-balance-mode)
 		       'omit-bal))
-		  (bal-method
-		   ;; figure out how to calculate our balance:
-		   ;; 'immediate-bal|'recursive-bal ('omit-bal handled below)
-		   (or (and (equal? row-type 'subtotal-row) 'recursive-bal)
-		       (and (equal? (+ display-depth 1) display-tree-depth)
-			    (or (and (equal? limit-behavior 'summarize)
-				     'recursive-bal)
-				(and (null? children) 'immediate-bal)
-				;; otherwise, parent account at depth limit,
-				;; with either 'truncate or 'flatten...
-				parent-acct-bal-mode
-				)
-			    )
-		       (if (null? children) #f parent-acct-bal-mode)
-		       'immediate-bal
-		       )
-		   )
-		  (comm-amt
-		   ;; this will be the immediate/recursive commodity
-		   ;; balance (a commodity collector) or #f.
-		   (get-val env
-			    (car (or (assoc-ref
-				      '((immediate-bal account-bal)
-					(recursive-bal recursive-bal)
-					(omit-bal #f))
-				      bal-method)
-				     '(#f)
-				     ))))
-		  (zero-mode (let ((mode
-				       (get-val
-					env 'zero-balance-display-mode)))
-			       (or (if (equal? mode #t) 'show-balance mode)
-				   'show-balance)
-			       ))
-		  (reverse-balance (gnc-reverse-balance acct))
-		  (native-comm?
-		   (lambda (amt)
-		     (gnc:uniform-commodity? amt report-commodity)))
-                  ;; amount is either a <gnc:monetary> or #f
-		  (amount (and comm-amt
-			       (if (and (equal? zero-mode 'omit-balance)
-                                        (gnc-commodity-collector-allzero?
-                                         comm-amt)
-                                        )
-				   #f
-				   ;; else:
-                                   ;; this let* block evals to a <gnc:monetary>
-				   (let*
-				       ((amt (gnc:make-commodity-collector)))
-				     (if reverse-balance
-					 (amt 'minusmerge comm-amt #f)
-					 (set! amt comm-amt))
-				     (or (and (native-comm? amt)
-					      (gnc:sum-collector-commodity
-					       amt
-					       report-commodity
-					       exchange-fn)
-					      )
-					 ((if (and (equal?
-                                                    multicommodity-mode 'table)
-                                                   (equal?
-                                                    row-type 'account-row)
-                                                   )
-                                              gnc-commodity-table
-                                              gnc:sum-collector-commodity
-                                              )
-                                          amt
-                                          report-commodity
-                                          exchange-fn
-                                          )  ;; factored from below
-; 					 (if (and (equal?
-; 						   multicommodity-mode 'table)
-; 						  (equal?
-; 						   row-type 'account-row)
-; 						  )
-; 					     (gnc-commodity-table
-; 					      amt
-; 					      report-commodity
-; 					      exchange-fn)
-; 					     (gnc:sum-collector-commodity
-; 					      amt
-; 					      report-commodity
-; 					      exchange-fn)
-; 					     )
 
-					 )
-				     ) ;; end of let*
-				   ) ;; end of if
-			       ))
-		  (indented-depth (get-val env 'indented-depth))
+                  (bal-method
+                   ;; figure out how to calculate our balance:
+                   ;; 'immediate-bal|'recursive-bal ('omit-bal handled below)
+                   (cond ((eq? row-type 'subtotal-row) 'recursive-bal)
+                         ((eq? (1+ display-depth) display-tree-depth)
+                          (cond ((eq? limit-behavior 'summarize) 'recursive-bal)
+                                ((null? children) 'immediate-bal)
+                                (else parent-acct-bal-mode)))
+                         ((not (null? children)) parent-acct-bal-mode)
+                         (else 'immediate-bal)))
+
+                  (comm-amt
+                   (get-val env (assq-ref '((immediate-bal . account-bal)
+                                            (recursive-bal . recursive-bal)
+                                            (omit-bal . #f))
+                                          bal-method)))
+                  (amt (and comm-amt
+                            (if (gnc-reverse-balance acct)
+                                (gnc:commodity-collector-get-negated comm-amt)
+                                comm-amt)))
+
+                  (zero-mode (let ((mode (get-val env 'zero-balance-display-mode)))
+                               (if (boolean? mode)
+                                   'show-balance
+                                   mode)))
+
+                  (native-comm?
+                   (lambda (amt)
+                     (gnc:uniform-commodity? amt report-commodity)))
+
+                  ;; amount is either a <gnc:monetary> or #f
+                  (amount (and amt
+                               (not (and (eq? zero-mode 'omit-balance)
+                                         (gnc-commodity-collector-allzero? amt)))
+                               (cond
+                                ((and (not (native-comm? amt))
+                                      (eq? multicommodity-mode 'table)
+                                      (eq? row-type 'account-row))
+                                 (gnc-commodity-table
+                                  amt report-commodity exchange-fn))
+                                (else
+                                 (gnc:sum-collector-commodity
+                                  amt report-commodity exchange-fn)))))
+
+                  (indented-depth (get-val env 'indented-depth))
 		  (account-colspan (get-val env 'account-colspan))
 		  )
 
@@ -1311,18 +1252,6 @@
 	      amount
 	      (+ account-cols (- 0 1)
 		 (- logical-cols display-depth)
-		 ;; account for 'immediate-bal parents displaying children
-		 ;; NOTE: before you go mucking with this, BE ABSOLUTELY
-		 ;; SURE you know what you're doing... i spent A LOT of
-		 ;; time trying to make sure this is right. i know, in
-		 ;; some reports, the output might look incorrect. but,
-		 ;; if you think long and hard about it, i think you'll
-		 ;; find the current treatment correct... i think. -DM-
-		 (- 0 (if (if (null? children)
-			      #f
-			      (equal? bal-method 'immediate-bal))
-			  1 0)
-		    )
 		 (if (equal? subtotal-mode 'canonically-tabbed) 1 0)
 		 )                          ;; amount-depth
 	      1                             ;; amount-colspan
@@ -1351,6 +1280,8 @@
   ;; presently unimplemented.  many of these functions are better left
   ;; to the renderer, anyway.  but if you *really* need them, you may
   ;; still use gnc:first-html-build-acct-table.
+  (issue-deprecation-warning
+   "gnc:second-html-build-acct-table is unused. use gnc:html-build-acct-table.")
   (let* ((env (list
 	       (list 'start-date start-date)
 	       (list 'end-date end-date)
